@@ -16,7 +16,7 @@ token_mapping = {
         "usdt": "0x55d398326f99059ff775485246999027b3197955",  # USDT on BSC
     },
     "base": {
-        "usdc": "0xd9f944a5c0c8ebBf5c72e50bd48b8C6dc76A07F4",  # USDC on Base
+        "usdc": "0x45b58118a5374dccf7fd6f3f66c66278ab7473d9",  # USDC on Base
     },
 }
 
@@ -155,8 +155,9 @@ control_contract_abi = [
 ]
 
 chain_id_mapping = {
-  'base': '0x2105',
-  'ethereum': '0x1',
+  'base': '0x14a34',
+  'ethereum': '0xaa36a7',
+  'eth': '0xaa36a7',
   'bsc': '0x38'
 }
 
@@ -204,7 +205,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
     private_key = kwargs.get('private_key')
     blockchain_explore = kwargs.get('blockchain_explore')
     providers = kwargs.get("providers")
-    provider = Web3.HTTPProvider(providers[chain])
+    provider = Web3.HTTPProvider(providers[chain.lower()])
     provider.middlewares.clear()
     w3 = Web3(provider)
     if run_mode == 'auto':
@@ -215,46 +216,72 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
       allowance_normalized = Web3.from_wei(allowance, 'mwei')
       print(f"allowance_normalized: {float(allowance_normalized)}")
       if float(allowance_normalized) < float(amount):
-          params = {
-            "func": "approve",
-            "chainId": chain_id_mapping[chain.lower()],
-            'contract': token,
-            'to': control_address,
-            'amount': float(amount),
-            'abi': erc20_approve_abi
-          }
-          action_data = ActionData(func='', group='', params=params)
-          action = Action(msg=None, type='wallet', data=action_data.__dict__)
+          active_chain_key = f"user:{executor}:active_chain"
+          active_chain = redis_client.get(active_chain_key)
+          if not active_chain:
+            active_chain = "base"
+          if active_chain.lower() != chain.lower():
+            action_params = {
+              "func": "wallet_switchEthereumChain",
+              "chainId": chain_id_mapping[chain.lower()]
+            }
+            action_data = ActionData(func='', group='', params=action_params)
+            action = Action(msg=None, type='setting', data=action_data.__dict__)
+            next_params = {
+              'receiver': receiver,
+              'amount': amount,
+              'chain': chain,
+              'token': token,
+              'run_mode': run_mode
+            }
+            next_action_data = ActionData(func='transfer', group='transfer', params=next_params)
+            next_action = Action(msg=None, type='backend', data=next_action_data.__dict__)
 
-          code, res = get_contact_address(redis_client, executor, receiver, w3)
-          if code != 200:
-              info = {"content": f"Cannot find {name} on contact", "code": 420}
-              return {
-                'result': info,
-                'action': None, 
-                'next_action': None 
-              }
-          value = float(amount)
-          receiver = res
-          to = w3.to_checksum_address(receiver)
-          params = {
-            'receiver': to,
-            'amount': amount,
-            'chain': chain,
-            'token': token,
-            'run_mode': run_mode
-          }
-          next_action_data = ActionData(func='transfer', group='transfer', params=params)
-          next_action = Action(msg=None, type='backend', data=next_action_data.__dict__)
-          return {
-            'result': { "code": 401, "content": "Insufficient allowance, proceeding with approval." },
-            'action': action.__dict__,
-            'next_action': next_action.__dict__
-          }
+            return {
+              'result': { 'code': 403, 'content': 'Incorrect network, switching to the correct network for you.' },
+              'action': action.__dict__,
+              'next_action': next_action.__dict__
+            }
+          else:
+            action_params = {
+              "func": "approve",
+              "chainId": chain_id_mapping[chain.lower()],
+              'contract': token,
+              'to': control_address,
+              'amount': float(amount),
+              'abi': erc20_approve_abi
+            }
+            action_data = ActionData(func='', group='', params=action_params)
+            action = Action(msg=None, type='wallet', data=action_data.__dict__)
 
+            code, res = get_contact_address(redis_client, executor, receiver, w3)
+            if code != 200:
+                info = {"content": f"Cannot find {receiver} on contact", "code": 420}
+                return {
+                  'result': info,
+                  'action': None, 
+                  'next_action': None 
+                }
+            value = float(amount)
+            receiver = res
+            to = w3.to_checksum_address(receiver)
+            next_params = {
+              'receiver': to,
+              'amount': amount,
+              'chain': chain,
+              'token': token,
+              'run_mode': run_mode
+            }
+            next_action_data = ActionData(func='transfer', group='transfer', params=next_params)
+            next_action = Action(msg=None, type='backend', data=next_action_data.__dict__)
+            return {
+              'result': { "code": 401, "content": "Insufficient allowance, proceeding with approval." },
+              'action': action.__dict__,
+              'next_action': next_action.__dict__
+            }
       code, res = get_contact_address(redis_client, executor, receiver, w3)
       if code != 200:
-          info = {"content": f"Cannot find {name} on contact", "code": 420}
+          info = {"content": f"Cannot find {receiver} on contact", "code": 420}
           return {
             'result': info,
             'action': None, 
@@ -329,7 +356,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
             "chainId": "ETH"
           }
           action_data = ActionData(func='', group='', params=params)
-          action = Action(msg=None, type='setting', data=action_data)
+          action = Action(msg=None, type='setting', data=action_data.__dict__)
 
           code, res = get_contact_address(redis_client, executor, receiver, w3)
           if code != 200:
@@ -349,7 +376,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
             'run_mode': run_mode
           }
           next_action_data = ActionData(func='transfer', group='transfer', params=params)
-          next_action = Action(msg=None, type='backend', data=next_action_data)
+          next_action = Action(msg=None, type='backend', data=next_action_data.__dict__)
 
           return {
             'result': { 'code': 403, 'content': 'Incorrect network, switching to the correct network for you.' },
@@ -377,7 +404,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
             'run_mode': run_mode
           }
           action_data = ActionData(func='transfer', group='transfer', params=params)
-          action = Action(msg=None, type='backend', data=action_data)
+          action = Action(msg=None, type='backend', data=action_data.__dict__)
           return {
             'result': None,
             'action': action.__dict__,
